library(rpart)

####### Part I - Credit card


credit<-read.csv('/Users/Thierry/Documents/R/Data/Economics/Greene/TableF7-3.csv',
                                   sep=",",header=TRUE)

n<-nrow(credit)
ntrain<-10000
ntest<-n-ntrain
train<-sample(n,ntrain)

credit$CARDHLDR<-as.factor(credit$CARDHLDR)


fit<-rpart(CARDHLDR~.-DEFAULT-SPENDING-LOGSPEND-EXP_INC,data=credit,
           subset=train,method="class",
           control = rpart.control(xval = 10, minbucket = 10,cp=0.00))

plot(fit,margin = 0.05)
text(fit,minlength=1,cex=0.8,splits=TRUE)

yhat=predict(fit,newdata=credit[-train,],type='class')
y.test=credit[-train,"CARDHLDR"]
table(y.test,yhat)
err<-1-mean(y.test==yhat)
print(err)

# Q3
plotcp(fit,minline=TRUE)
printcp(fit)

# Value of alpha with minimum cross-validation error
i.min<-which.min(fit$cptable[,4])
cp.opt1<-fit$cptable[i.min,1]
# Value of alpha according to the 1se rule
x<-fit$cptable[i.min,4]+fit$cptable[i.min,5]
ii<-which(fit$cptable[,4]<=x)
cp.opt2<-fit$cptable[min(ii),1]


pruned_tree<-prune(fit,cp=cp.opt2)
plot(pruned_tree,margin = 0.1)
text(pruned_tree,pretty=0)

yhat=predict(pruned_tree,newdata=credit[-train,],type='class')
CM<-table(y.test,yhat1)
err<-1-mean(y.test==yhat1)
print(err)

# Q4
library(pROC)
prob<-predict(pruned_tree,newdata=credit[-train,],type='prob')
roc_tree<-roc(y.test,prob[,1])
plot(roc_tree)
TPR<-CM[2,2]/rowSums(CM)[2]
FPR<-CM[1,2]/rowSums(CM)[1]
points(1-FPR,TPR)


# Q5

# Bagging
library(randomForest)
credit1<-credit[,-c(2,12:14)]
p<-ncol(credit1)-1
fit.bag=randomForest(CARDHLDR~.,data=credit1,subset=train,mtry=p)

yhat1=predict(fit.bag,newdata=credit1[-train,],type="response")
CM1<-table(y.test,yhat1)
1-mean(y.test==yhat1)

prob1<-predict(fit.bag,newdata=credit1[-train,],type="prob")
roc_tree1<-roc(y.test,prob1[,1])
plot(roc_tree1,add=TRUE,col="red")
TPR<-CM1[2,2]/rowSums(CM1)[2]
FPR<-CM1[1,2]/rowSums(CM1)[1]
points(1-FPR,TPR,col="red")

# Random Forest
fit.rf=randomForest(CARDHLDR~.,data=credit1,subset=train)

yhat2=predict(fit.rf,newdata=credit1[-train,],type="response")
CM2<-table(y.test,yhat2)
1-mean(y.test==yhat2)

prob2<-predict(fit.rf,newdata=credit1[-train,],type="prob")
roc_tree2<-roc(y.test,prob2[,1])
plot(roc_tree2,add=TRUE,col="green")
TPR<-CM2[2,2]/rowSums(CM2)[2]
FPR<-CM2[1,2]/rowSums(CM2)[1]
points(1-FPR,TPR,col="green")


# Q6

# Logistic regression
fit.lr<-glm(CARDHLDR~.,data=credit1,subset=train,family="binomial")
pred.lr=predict(fit.lr,newdata=credit1[-train,],type="response")
yhat3<-pred.lr>0.5
CM3<-table(y.test,yhat3)
1-sum(diag(CM3))/(n-ntrain)

roc_tree3<-roc(y.test,pred.lr)
plot(roc_tree3,add=TRUE,col="magenta")

summary(fit.lr)

# GAM
library(gam)
fit.gam<-gam(CARDHLDR ~ s(AGE)+s(ACADMOS)+s(ADEPCNT)+s(MAJORDRG)+
           s(MINORDRG)+OWNRENT+s(INCOME)+SELFEMPL+s(INCPER),
         data=credit1,subset=train,family='binomial',trace=TRUE)

plot(fit.gam,residuals=TRUE)

pred.gam=predict(fit.gam,newdata=credit1[-train,],type="response")
yhat4<-pred.gam>0.5
CM4<-table(y.test,yhat4)
1-sum(diag(CM4))/(n-ntrain)

roc_tree4<-roc(y.test,pred.gam)
plot(roc_tree4,add=TRUE,col="yellow")


####### Exercise 2: Boston



library(MASS)  
n<-nrow(Boston)
ntrain<-round(2*n/3)
ntest<-n-ntrain
train<-sample(n,ntrain)

# Q1
tree.boston<-rpart(medv~.,data=Boston,subset=train,
                   method="anova",
           control = rpart.control(xval = 10, minbucket = 10, cp = 0))

plot(tree.boston,margin=0.05)
text(tree.boston,pretty=0)

pred.tree<-predict(tree.boston,newdata=Boston[-train,])
y.test<-Boston$medv[-train]
mse.tree<-mean((y.test-pred.tree)^2)
print(mse.tree)


printcp(tree.boston)
plotcp(tree.boston)

# Value of alpha with minimum cross-validation error
i.min<-which.min(tree.boston$cptable[,4])
cp.opt1<-tree.boston$cptable[i.min,1]
# Value of alpha according to the 1se rule
x<-tree.boston$cptable[i.min,4]+tree.boston$cptable[i.min,5]
ii<-which(tree.boston$cptable[,4]<=x)
cp.opt2<-tree.boston$cptable[min(ii),1]



pruned_tree<-prune(tree.boston,cp=cp.opt1)
plot(pruned_tree,margin=0.05)
text(pruned_tree,pretty=0)

pred.tree<-predict(pruned_tree,newdata=Boston[-train,])
y.test<-Boston$medv[-train]
mse.tree<-mean((y.test-pred.tree)^2)
print(mse.tree)

plot(y.test,pred.tree)

# Q2
# Bagging

p<-ncol(Boston)-1
fit.bag<-randomForest(medv~.,data=Boston,subset=train,mtry=p)
pred.bag<-predict(fit.bag,newdata=Boston[-train,])
mse.bag<-mean((y.test-pred.bag)^2)
print(mse.bag)

# Random forest

fit.rf<-randomForest(medv~.,data=Boston,subset=train)
pred.rf<-predict(fit.rf,newdata=Boston[-train,])
mse.rf<-mean((y.test-pred.rf)^2)
print(mse.rf)
points(y.test,pred.rf,col="red",pch=2)

# Q3

# LS regression
reg<- lm(medv ~. ,data=Boston,subset=train)
pred<-predict(reg,newdata=Boston[-train,])
mse.ls<-mean((y.test-pred)^2)
print(mse.ls)

# Ridge
library(glmnet)
xapp<-as.matrix(Boston[train,-14])
xtst<-as.matrix(Boston[-train,-14])
yapp<-Boston$medv[train]
cv.out<-cv.glmnet(xapp,yapp,alpha=0,standardize=TRUE)
fit<-glmnet(xapp,yapp,lambda=cv.out$lambda.min,alpha=0,standardize=TRUE)
ridge.pred<-predict(fit,s=cv.out$lambda.min,newx=xtst)
mse.ridge<-mean((y.test-ridge.pred)^2)

# Lasso
cv.out<-cv.glmnet(xapp,yapp,alpha=1,standardize=TRUE)
fit<-glmnet(xapp,yapp,lambda=cv.out$lambda.min,alpha=1,standardize=TRUE)
lasso.pred<-predict(fit,s=cv.out$lambda.min,newx=xtst)
mse.lasso<-mean((y.test-lasso.pred)^2)

print(c(mse.tree,mse.bag,mse.rf,mse.ls, mse.ridge,mse.lasso))

# Subset selection
library(leaps)
reg.forward<-regsubsets(medv~.,data=Boston, subset=train,
                        method='forward',nvmax=30)
plot(reg.forward,scale="bic")
res<-summary(reg.forward)

best<-which.min(res$bic)
ntst<-nrow(xtst)
X<-cbind(rep(1,ntst),xtst)

ypred<-X[,res$which[best,]]%*%coef(reg.forward,best)
mse.forward.bic<-mean((ypred-y.test)^2)

print(c(mse.tree,mse.ls, mse.ridge,mse.lasso,mse.forward.bic))

# Q4

# GAM with all variables
fit.gam<-gam(medv ~ s(crim)+s(zn)+s(indus)+chas+s(rm)+s(age)+s(rad)+s(tax)+
               s(ptratio)+s(black)+s(lstat) + s(dis) + s(nox),
             subset=train,data=Boston,trace=TRUE)

pred.gam<-predict(fit.gam,newdata=Boston[-train,])
mse.gam<-mean((y.test-pred.gam)^2)
print(mse.gam)

# GAM with variables selected by forward selection
fit.gam1<-gam(medv ~ chas+s(rm)+
               s(ptratio)+s(black)+s(lstat) + s(dis) + s(nox),
             subset=train,data=Boston,trace=TRUE)
pred.gam1<-predict(fit.gam1,newdata=Boston[-train,])
mse.gam1<-mean((y.test-pred.gam1)^2)
print(mse.gam1)


